@@ -39,10 +39,12 @@
 
 #include "getdns/getdns.h"
 #include "getdns/getdns_extra.h"
+#include "config.h"
 #include "types-internal.h"
+#include "extension/libmini_event.h"
+#include "util/rbtree.h"
 
 struct getdns_dns_req;
-struct ldns_rbtree_t;
 struct ub_ctx;
 
 #define GETDNS_FN_RESOLVCONF "/etc/resolv.conf"
@@ -66,8 +68,37 @@ struct filechg {
 	struct stat *prevstat;
 };
 
-struct getdns_context {
+typedef struct getdns_upstream {
+	struct getdns_upstreams *upstreams;
 
+	socklen_t                addr_len;
+	struct sockaddr_storage  addr;
+
+	/* How is this upstream doing? */
+	int                      to_retry;
+	int                      back_off;
+
+	/* For sharing a TCP socket to this upstream */
+	int                      fd;
+	getdns_eventloop_event   event;
+	getdns_eventloop        *loop;
+	getdns_tcp_state         tcp;
+
+	/* Pipelining of TCP network requests */
+	getdns_network_req      *write_queue;
+	getdns_network_req      *write_queue_last;
+	getdns_rbtree_t          netreq_by_query_id;
+} getdns_upstream;
+
+typedef struct getdns_upstreams {
+	struct mem_funcs mf;
+	size_t referenced;
+	size_t count;
+	size_t current;
+	getdns_upstream upstreams[];
+} getdns_upstreams;
+
+struct getdns_context {
 	/* Context values */
 	getdns_resolution_t  resolution_type;
 	getdns_namespace_t   *namespaces;
@@ -78,29 +109,31 @@ struct getdns_context {
 	getdns_append_name_t append_name;
 	struct getdns_list   *suffix;
 	struct getdns_list   *dnssec_trust_anchors;
-	struct getdns_list   *upstream_list;
-    getdns_transport_t   dns_transport;
-    uint16_t             limit_outstanding_queries;
-    uint32_t             dnssec_allowed_skew;
+	getdns_upstreams     *upstreams;
+	getdns_transport_t   dns_transport;
+	uint16_t             limit_outstanding_queries;
+	uint32_t             dnssec_allowed_skew;
 
 	uint8_t edns_extended_rcode;
 	uint8_t edns_version;
 	uint8_t edns_do_bit;
-    uint16_t edns_maximum_udp_payload_size;
+	int edns_maximum_udp_payload_size; /* -1 is unset */
 
 	getdns_update_callback update_callback;
 
-    int processing;
-    int destroying;
+	int processing;
+	int destroying;
 
 	struct mem_funcs mf;
 	struct mem_funcs my_mf;
 
-	/* The underlying unbound contexts that do
-	 * the real work */
+	/* The underlying contexts that do the real work */
 	struct ub_ctx *unbound_ctx;
+
+	/* A tree to hold local host information*/
+	getdns_rbtree_t local_hosts;
 	int has_ta; /* No DNSSEC without trust anchor */
-    int return_dnssec_status;
+	int return_dnssec_status;
 
 	/* which resolution type the contexts are configured for
 	 * 0 means nothing set
@@ -110,27 +143,14 @@ struct getdns_context {
 	/*
 	 * outbound requests -> transaction to getdns_dns_req
 	 */
-	struct ldns_rbtree_t *outbound_requests;
+	getdns_rbtree_t outbound_requests;
 
-    /*
-     * Event loop extension functions
-     * These structs are static and should never be freed
-     * since they are just a collection of function pointers
-     */
-    getdns_eventloop_extension* extension;
-    /*
-     * Extension data that will be freed by the functions
-     * in the extension struct
-     */
-    void* extension_data;
+	/* Event loop extension.  */
+	getdns_eventloop       *extension;
+	getdns_eventloop_event  ub_event;
 
-    /*
-     * Timeout info one tree to manage timeout data
-     * keyed by transaction id.  Second to manage by
-     * timeout time (ascending)
-     */
-    struct ldns_rbtree_t *timeouts_by_id;
-    struct ldns_rbtree_t *timeouts_by_time;
+	/* The default extension */
+	getdns_mini_event mini_event;
 
 	/*
 	 * state data used to detect changes to the system config files
@@ -175,14 +195,12 @@ void getdns_bindata_destroy(
     struct mem_funcs *mfs,
     struct getdns_bindata *bindata);
 
-/* timeout scheduling */
-getdns_return_t getdns_context_schedule_timeout(struct getdns_context* context,
-    getdns_transaction_t id, uint16_t timeout, getdns_timeout_callback callback,
-    void* userarg);
-
-getdns_return_t getdns_context_clear_timeout(struct getdns_context* context,
-    getdns_transaction_t id);
+/* perform name resolution in /etc/hosts */
+getdns_return_t getdns_context_local_namespace_resolve(
+    getdns_dns_req* req, struct getdns_dict **response);
 
 int filechg_check(struct getdns_context *context, struct filechg *fchg);
 
+void priv_getdns_context_ub_read_cb(void *userarg);
+
 #endif /* _GETDNS_CONTEXT_H_ */
