@@ -1,7 +1,7 @@
 /**
  *
  * getdns dict management functions, note that the internal storage is
- * accomplished via an ldns_rbtree_t
+ * accomplished via an getdns_rbtree_t
  *
  * Interfaces originally taken from the getdns API description pseudo implementation.
  *
@@ -59,7 +59,7 @@ struct getdns_dict_item *
 getdns_dict_find(const struct getdns_dict *dict, const char *key)
 {
 	return (struct getdns_dict_item *)
-		   ldns_rbtree_search((ldns_rbtree_t *)&(dict->root), key);
+		   getdns_rbtree_search((getdns_rbtree_t *)&(dict->root), key);
 }				/* getdns_dict_find */
 
 struct getdns_dict_item *
@@ -68,14 +68,14 @@ getdns_dict_find_and_add(struct getdns_dict *dict, const char *key)
 	struct getdns_dict_item *item;
 
 	item = (struct getdns_dict_item *)
-		   ldns_rbtree_search(&(dict->root), key);
+		   getdns_rbtree_search(&(dict->root), key);
 
 	if (!item) {
 		/* add a node */
 		item = GETDNS_MALLOC(dict->mf, struct getdns_dict_item);
 		item->node.key = getdns_strdup(&dict->mf, key);
 		item->data.n = 0;
-		ldns_rbtree_insert(&(dict->root), (ldns_rbnode_t *) item);
+		getdns_rbtree_insert(&(dict->root), (getdns_rbnode_t *) item);
 	}
 	return item;
 }				/* getdns_dict_find_and_add */
@@ -100,8 +100,8 @@ getdns_dict_get_names(const struct getdns_dict * dict,
 	if (!*answer)
 		return GETDNS_RETURN_NO_SUCH_DICT_NAME;
 
-	LDNS_RBTREE_FOR(item, struct getdns_dict_item *,
-		(ldns_rbtree_t *)&(dict->root)) {
+	RBTREE_FOR(item, struct getdns_dict_item *,
+		(getdns_rbtree_t *)&(dict->root)) {
 		if (getdns_list_add_item(*answer, &index) != GETDNS_RETURN_GOOD)
 			continue;
 		bindata.size = strlen(item->node.key) + 1;
@@ -240,7 +240,7 @@ getdns_dict_create_with_extended_memory_functions(
 	dict->mf.mf.ext.realloc = realloc;
 	dict->mf.mf.ext.free    = free;
 
-	ldns_rbtree_init(&(dict->root),
+	getdns_rbtree_init(&(dict->root),
 		(int (*)(const void *, const void *)) strcmp);
 	return dict;
 }
@@ -310,8 +310,8 @@ getdns_dict_copy(const struct getdns_dict * srcdict,
 		return GETDNS_RETURN_GENERIC_ERROR;
 
 	retval = GETDNS_RETURN_GOOD;
-	LDNS_RBTREE_FOR(item, struct getdns_dict_item *,
-		(struct ldns_rbtree_t *)&(srcdict->root)) {
+	RBTREE_FOR(item, struct getdns_dict_item *,
+		(struct getdns_rbtree_t *)&(srcdict->root)) {
 		key = (char *) item->node.key;
 		switch (item->dtype) {
 		case t_bindata:
@@ -352,7 +352,7 @@ getdns_dict_copy(const struct getdns_dict * srcdict,
  * @return void
  */
 void
-getdns_dict_item_free(ldns_rbnode_t * node, void *arg)
+getdns_dict_item_free(getdns_rbnode_t * node, void *arg)
 {
 	struct getdns_dict_item *item = (struct getdns_dict_item *) node;
 	struct getdns_dict *dict = (struct getdns_dict *)arg;
@@ -382,11 +382,8 @@ getdns_dict_item_free(ldns_rbnode_t * node, void *arg)
 void
 getdns_dict_destroy(struct getdns_dict *dict)
 {
-	if (!dict)
-			return;
-
-	ldns_traverse_postorder(&(dict->root),
-		getdns_dict_item_free, dict);
+	if (!dict) return;
+	getdns_traverse_postorder(&(dict->root), getdns_dict_item_free, dict);
 	GETDNS_FREE(dict->mf, dict);
 }				/* getdns_dict_destroy */
 
@@ -584,11 +581,14 @@ getdns_pp_dict(ldns_buffer * buf, size_t indent,
  * @param buf    buffer to write to
  * @param indent number of spaces to append after newline
  * @param list   the to list print
+ * @param for_namespaces The list is a list of namespace literals.
+ *               Show the literal instead of the value.
  * @return       on success the number of written characters
  *               if an output error is encountered, a negative value
  */
 static int
-getdns_pp_list(ldns_buffer * buf, size_t indent, struct getdns_list *list)
+getdns_pp_list(ldns_buffer *buf, size_t indent, getdns_list *list,
+    int for_namespaces)
 {
 	size_t i, length, p = ldns_buffer_position(buf);
 	getdns_data_type dtype;
@@ -596,6 +596,7 @@ getdns_pp_list(ldns_buffer * buf, size_t indent, struct getdns_list *list)
 	struct getdns_list *list_item;
 	struct getdns_bindata *bindata_item;
 	uint32_t int_item;
+	const char *strval;
 
 	if (list == NULL)
 		return 0;
@@ -618,9 +619,16 @@ getdns_pp_list(ldns_buffer * buf, size_t indent, struct getdns_list *list)
 
 		switch (dtype) {
 		case t_int:
-			if (getdns_list_get_int(list, i, &int_item) !=
-				GETDNS_RETURN_GOOD ||
-				ldns_buffer_printf(buf, "%d", (int) int_item) < 0)
+
+			if (getdns_list_get_int(list, i, &int_item))
+				return -1;
+			if (for_namespaces &&
+			    (strval =
+			     priv_getdns_get_const_info(int_item)->name)) {
+				if (ldns_buffer_printf(buf, "%s", strval) < 0)
+					return -1;
+			} else if (0 > 
+			    ldns_buffer_printf(buf, "%d", (int)int_item))
 				return -1;
 			break;
 
@@ -636,7 +644,7 @@ getdns_pp_list(ldns_buffer * buf, size_t indent, struct getdns_list *list)
 			if (getdns_list_get_list(list, i, &list_item) !=
 				GETDNS_RETURN_GOOD)
 				return -1;
-			if (getdns_pp_list(buf, indent, list_item) < 0)
+			if (getdns_pp_list(buf, indent, list_item, 0) < 0)
 				return -1;
 			break;
 
@@ -758,8 +766,8 @@ getdns_pp_dict(ldns_buffer * buf, size_t indent,
 
 	i = 0;
 	indent += 2;
-	LDNS_RBTREE_FOR(item, struct getdns_dict_item *,
-		(ldns_rbtree_t *)&(dict->root)) {
+	RBTREE_FOR(item, struct getdns_dict_item *,
+		(getdns_rbtree_t *)&(dict->root)) {
 		if (ldns_buffer_printf(buf, "%s\n%s\"%s\":", (i ? "," : "")
 			, getdns_indent(indent)
 			, item->node.key) < 0)
@@ -768,31 +776,35 @@ getdns_pp_dict(ldns_buffer * buf, size_t indent,
 		switch (item->dtype) {
 		case t_int:
 			if ((strcmp(item->node.key, "type") == 0  ||
-				 strcmp(item->node.key, "type_covered") == 0 ||
-				 strcmp(item->node.key, "qtype") == 0) &&
-				(strval = priv_getdns_rr_type_name(item->data.n))) {
+			     strcmp(item->node.key, "type_covered") == 0 ||
+			     strcmp(item->node.key, "qtype") == 0) &&
+			    (strval = priv_getdns_rr_type_name(item->data.n))) {
 				if (ldns_buffer_printf(
 					buf, " GETDNS_RRTYPE_%s", strval) < 0)
 					return -1;
 				break;
 			}
-				if ((strcmp(item->node.key, "answer_type") == 0  ||
-				 strcmp(item->node.key, "dnssec_status") == 0 ||
-				 strcmp(item->node.key, "status") == 0) &&
-				(strval =
-				 priv_getdns_get_const_info(item->data.n)->name)) {
+			if ((strcmp(item->node.key, "answer_type") == 0  ||
+			     strcmp(item->node.key, "dnssec_status") == 0 ||
+			     strcmp(item->node.key, "status") == 0 ||
+			     strcmp(item->node.key, "append_name") == 0 ||
+			     strcmp(item->node.key, "dns_transport") == 0 ||
+			     strcmp(item->node.key, "follow_redirects") == 0 ||
+			     strcmp(item->node.key, "resolution_type") == 0) &&
+			    (strval =
+			     priv_getdns_get_const_info(item->data.n)->name)) {
 				if (ldns_buffer_printf(buf, " %s", strval) < 0)
 					return -1;
 				break;
 			}
-				if ((strcmp(item->node.key, "class")  == 0  ||
+			if ((strcmp(item->node.key, "class")  == 0  ||
 				 strcmp(item->node.key, "qclass") == 0) &&
 				priv_getdns_print_class(buf, item->data.n))
 				break;
-				if (strcmp(item->node.key, "opcode") == 0 &&
+			if (strcmp(item->node.key, "opcode") == 0 &&
 				priv_getdns_print_opcode(buf, item->data.n))
 				break;
-				if (strcmp(item->node.key, "rcode") == 0 &&
+			if (strcmp(item->node.key, "rcode") == 0 &&
 				priv_getdns_print_rcode(buf, item->data.n))
 				break;
 			if (ldns_buffer_printf(buf, " %d", item->data.n) < 0)
@@ -833,7 +845,8 @@ getdns_pp_dict(ldns_buffer * buf, size_t indent,
 			if (ldns_buffer_printf(buf, "\n%s",
 				getdns_indent(indent)) < 0)
 				return -1;
-			if (getdns_pp_list(buf, indent, item->data.list) < 0)
+			if (getdns_pp_list(buf, indent, item->data.list, 
+			    (strcmp(item->node.key, "namespaces") == 0)) < 0)
 				return -1;
 			break;
 
@@ -902,7 +915,7 @@ getdns_dict_remove_name(struct getdns_dict *this_dict, const char *name)
 		return GETDNS_RETURN_NO_SUCH_DICT_NAME;
 
 	/* cleanup */
-	ldns_rbtree_delete(&this_dict->root, name);
+	getdns_rbtree_delete(&this_dict->root, name);
 	getdns_dict_item_free(&item->node, this_dict);
 
 	return GETDNS_RETURN_GOOD;
