@@ -47,16 +47,14 @@
 #include "types-internal.h"
 #include "dnssec.h"
 #include "rr-dict.h"
-#include "ub_timed_resolve.h"
 
 void priv_getdns_call_user_callback(getdns_dns_req *, struct getdns_dict *);
 
 struct validation_chain {
-	ldns_rbtree_t root;
+	getdns_rbtree_t root;
 	struct mem_funcs mf;
 	getdns_dns_req *dns_req;
 	size_t lock;
-	struct getdns_dict **sync_response;
 	uint64_t *timeout;
 };
 
@@ -70,7 +68,7 @@ struct chain_response {
 };
 
 struct chain_link {
-	ldns_rbnode_t node;
+	getdns_rbnode_t node;
 	struct chain_response DNSKEY;
 	struct chain_response DS;
 };
@@ -87,8 +85,8 @@ static void callback_on_complete_chain(struct validation_chain *chain)
 	ldns_rr_list *keys;
 	struct getdns_list *getdns_keys;
 
-	LDNS_RBTREE_FOR(link, struct chain_link *,
-	    (ldns_rbtree_t *)&(chain->root)) {
+	RBTREE_FOR(link, struct chain_link *,
+	    (getdns_rbtree_t *)&(chain->root)) {
 		if (link->DNSKEY.result == NULL && link->DNSKEY.err == 0)
 			ongoing++;
 		if (link->DS.result     == NULL && link->DS.err     == 0 &&
@@ -101,8 +99,8 @@ static void callback_on_complete_chain(struct validation_chain *chain)
 		response = create_getdns_response(chain->dns_req);
 
 		keys = ldns_rr_list_new();
-		LDNS_RBTREE_FOR(link, struct chain_link *,
-		    (ldns_rbtree_t *)&(chain->root)) {
+		RBTREE_FOR(link, struct chain_link *,
+		    (getdns_rbtree_t *)&(chain->root)) {
 			(void) ldns_rr_list_cat(keys, link->DNSKEY.result);
 			(void) ldns_rr_list_cat(keys, link->DS.result);
 		}
@@ -111,10 +109,7 @@ static void callback_on_complete_chain(struct validation_chain *chain)
 		    getdns_keys);
 		getdns_list_destroy(getdns_keys);
 		ldns_rr_list_free(keys);
-		if (chain->sync_response) {
-			*chain->sync_response = response;
-		} else
-			priv_getdns_call_user_callback(dns_req, response);
+		priv_getdns_call_user_callback(dns_req, response);
 		destroy_chain(chain);
 	}
 }
@@ -197,20 +192,10 @@ static void chain_response_init(
 static int
 resolve(char* name, int rrtype, struct chain_response *response)
 {
-	int r;
-	struct ub_result *ub_res;
-
-	if (response->chain->sync_response) {
-		ub_res = NULL;
-		r = ub_timed_resolve(response->chain->dns_req->context->unbound_ctx,
-		    name, rrtype, LDNS_RR_CLASS_IN, &ub_res, response->chain->timeout);
-		ub_chain_response_callback(response, r, ub_res);
-		return r;
-	} else
-		return ub_resolve_async(
-		    response->chain->dns_req->context->unbound_ctx,
-		    name, rrtype, LDNS_RR_CLASS_IN, response,
-		    ub_chain_response_callback, &response->unbound_id);
+	return ub_resolve_async(
+	    response->chain->dns_req->context->unbound_ctx,
+	    name, rrtype, LDNS_RR_CLASS_IN, response,
+	    ub_chain_response_callback, &response->unbound_id);
 }
 
 static void
@@ -218,7 +203,7 @@ launch_chain_link_lookup(struct validation_chain *chain, char *name)
 {
 	int r;
 	struct chain_link *link = (struct chain_link *)
-	    ldns_rbtree_search((ldns_rbtree_t *)&(chain->root), name);
+	    getdns_rbtree_search((getdns_rbtree_t *)&(chain->root), name);
 
 	if (link) {
 		free(name);
@@ -230,7 +215,7 @@ launch_chain_link_lookup(struct validation_chain *chain, char *name)
 	chain_response_init(chain, &link->DNSKEY);
 	chain_response_init(chain, &link->DS);
 
-	ldns_rbtree_insert(&(chain->root), (ldns_rbnode_t *)link);
+	getdns_rbtree_insert(&(chain->root), (getdns_rbnode_t *)link);
 
 	chain->lock++;
 	r = resolve(name, LDNS_RR_TYPE_DNSKEY, &link->DNSKEY);
@@ -246,7 +231,7 @@ launch_chain_link_lookup(struct validation_chain *chain, char *name)
 }
 
 static struct validation_chain *create_chain(getdns_dns_req *dns_req,
-    struct getdns_dict **sync_response, uint64_t *timeout)
+    uint64_t *timeout)
 {
 	struct validation_chain *chain = GETDNS_MALLOC(
 	    dns_req->context->mf, struct validation_chain);
@@ -254,7 +239,7 @@ static struct validation_chain *create_chain(getdns_dns_req *dns_req,
 	if (! chain)
 		return NULL;
 
-	ldns_rbtree_init(&(chain->root),
+	getdns_rbtree_init(&(chain->root),
 	    (int (*)(const void *, const void *)) strcmp);
 	chain->mf.mf_arg         = dns_req->context->mf.mf_arg;
 	chain->mf.mf.ext.malloc  = dns_req->context->mf.mf.ext.malloc;
@@ -262,12 +247,11 @@ static struct validation_chain *create_chain(getdns_dns_req *dns_req,
 	chain->mf.mf.ext.free    = dns_req->context->mf.mf.ext.free;
 	chain->dns_req = dns_req;
 	chain->lock = 0;
-	chain->sync_response = sync_response;
 	chain->timeout = timeout;
 	return chain;
 }
 
-static void destroy_chain_link(ldns_rbnode_t * node, void *arg)
+static void destroy_chain_link(getdns_rbnode_t * node, void *arg)
 {
 	struct chain_link *link = (struct chain_link*) node;
 	struct validation_chain *chain   = (struct validation_chain*) arg;
@@ -280,26 +264,20 @@ static void destroy_chain_link(ldns_rbnode_t * node, void *arg)
 
 static void destroy_chain(struct validation_chain *chain)
 {
-	ldns_traverse_postorder(&(chain->root),
-	    destroy_chain_link, chain);
+	getdns_traverse_postorder(&(chain->root), destroy_chain_link, chain);
 	GETDNS_FREE(chain->mf, chain);
 }
 
 /* Do some additional requests to fetch the complete validation chain */
 static void
-getdns_get_validation_chain(getdns_dns_req *dns_req,
-    struct getdns_dict **sync_response, uint64_t *timeout)
+getdns_get_validation_chain(getdns_dns_req *dns_req, uint64_t *timeout)
 {
 	getdns_network_req *netreq = dns_req->first_req;
-	struct validation_chain *chain = create_chain(
-	    dns_req, sync_response, timeout);
+	struct validation_chain *chain = create_chain(dns_req, timeout);
 
 	if (! chain) {
-		if (sync_response)
-			*sync_response = create_getdns_response(dns_req);
-		else
-			priv_getdns_call_user_callback(
-			    dns_req, create_getdns_response(dns_req));
+		priv_getdns_call_user_callback(
+		    dns_req, create_getdns_response(dns_req));
 		return;
 	}
 	while (netreq) {
@@ -326,15 +304,7 @@ getdns_get_validation_chain(getdns_dns_req *dns_req,
 
 void priv_getdns_get_validation_chain(getdns_dns_req *dns_req)
 {
-	getdns_get_validation_chain(dns_req, NULL, NULL);
-}
-
-struct getdns_dict *
-priv_getdns_get_validation_chain_sync(getdns_dns_req *dns_req, uint64_t *timeout)
-{
-	struct getdns_dict *sync_response = NULL;
-	getdns_get_validation_chain(dns_req, &sync_response, timeout);
-	return sync_response;
+	getdns_get_validation_chain(dns_req, NULL);
 }
 
 /********************** functions for validate_dnssec *************************/
@@ -687,7 +657,6 @@ getdns_root_trust_anchor(time_t *utc_date_of_anchor)
 
 	if (! priv_getdns_parse_ta_file(utc_date_of_anchor, tas_rr_list)) {
 		goto done_free_tas_rr_list;
-		return NULL;
 	}
 	tas_gd_list = create_list_from_rr_list(NULL, tas_rr_list);
 
