@@ -36,6 +36,7 @@
 #include "config.h"
 #include "types-internal.h"
 #include "util-internal.h"
+#include "gldns/rrdef.h"
 
 void
 network_req_free(getdns_network_req * net_req)
@@ -71,6 +72,13 @@ network_req_new(getdns_dns_req * owner,
 
 	/* TODO: records and other extensions */
 
+	net_req->upstream = NULL;
+	net_req->fd = -1;
+	memset(&net_req->event, 0, sizeof(net_req->event));
+	memset(&net_req->tcp, 0, sizeof(net_req->tcp));
+	net_req->query_id = 0;
+	net_req->max_udp_payload_size = 0;
+	net_req->write_queue_tail = NULL;
 	return net_req;
 }
 
@@ -81,11 +89,13 @@ dns_req_free(getdns_dns_req * req)
 		return;
 	}
 	getdns_network_req *net_req = NULL;
-	struct getdns_context *context = req->context;
 
 	/* free extensions */
 	getdns_dict_destroy(req->extensions);
 
+	if (req->upstreams && --req->upstreams->referenced == 0)
+		GETDNS_FREE(req->upstreams->mf, req->upstreams);
+
 	/* free network requests */
 	net_req = req->first_req;
 	while (net_req) {
@@ -94,56 +104,54 @@ dns_req_free(getdns_dns_req * req)
 		net_req = next;
 	}
 
-    if (req->local_timeout_id != 0) {
-        getdns_context_clear_timeout(context, req->local_timeout_id);
-    }
-
-    getdns_context_clear_timeout(context, req->trans_id);
+	if (req->timeout.timeout_cb) {
+		req->loop->vmt->clear(req->loop, &req->timeout);
+		req->timeout.timeout_cb = NULL;
+	}
 
 	/* free strduped name */
 	GETDNS_FREE(req->my_mf, req->name);
-
 	GETDNS_FREE(req->my_mf, req);
 }
 
 /* create a new dns req to be submitted */
 getdns_dns_req *
-dns_req_new(struct getdns_context *context,
+dns_req_new(struct getdns_context *context, getdns_eventloop *loop,
     const char *name, uint16_t request_type, struct getdns_dict *extensions)
 {
 
 	getdns_dns_req *result = NULL;
 	getdns_network_req *req = NULL;
-
-        getdns_return_t r;
-        uint32_t klass;
+        uint32_t klass = GLDNS_RR_CLASS_IN;
         
 	result = GETDNS_MALLOC(context->mf, getdns_dns_req);
 	if (result == NULL) {
 		return NULL;
 	}
-    result->my_mf = context->mf;
+	result->my_mf = context->mf;
 	result->name = getdns_strdup(&(result->my_mf), name);
 	result->context = context;
+	result->loop = loop;
 	result->canceled = 0;
 	result->current_req = NULL;
 	result->first_req = NULL;
-	result->trans_id = ldns_get_random();
+	result->trans_id = (uint64_t)(((intptr_t) result) ^ ldns_get_random());
 
 	getdns_dict_copy(extensions, &result->extensions);
-    result->return_dnssec_status = context->return_dnssec_status;
+	result->return_dnssec_status = context->return_dnssec_status;
 
 	/* will be set by caller */
 	result->user_pointer = NULL;
 	result->user_callback = NULL;
-    result->local_timeout_id = 0;
+	memset(&result->timeout, 0, sizeof(result->timeout));
 
         /* check the specify_class extension */
-        if ((r = getdns_dict_get_int(extensions, "specify_class", &klass))
-                != GETDNS_RETURN_GOOD) {
-            klass = LDNS_RR_CLASS_IN;
-        }
+        (void) getdns_dict_get_int(extensions, "specify_class", &klass);
         
+	result->upstreams = context->upstreams;
+	if (result->upstreams)
+		result->upstreams->referenced++;
+
 	/* create the requests */
 	req = network_req_new(result, request_type, klass, extensions);
 	if (!req) {
@@ -159,13 +167,11 @@ dns_req_new(struct getdns_context *context,
 	    (request_type == GETDNS_RRTYPE_A ||
 	     request_type == GETDNS_RRTYPE_AAAA)) {
 
-		uint16_t next_req_type =
-		    (request_type ==
-		    GETDNS_RRTYPE_A) ? GETDNS_RRTYPE_AAAA : GETDNS_RRTYPE_A;
+		uint16_t next_req_type = (request_type == GETDNS_RRTYPE_A) ?
+		    GETDNS_RRTYPE_AAAA : GETDNS_RRTYPE_A;
 		getdns_network_req *next_req = network_req_new(result,
-		    next_req_type,
-		    LDNS_RR_CLASS_IN,
-		    extensions);
+		    next_req_type, LDNS_RR_CLASS_IN, extensions);
+
 		if (!next_req) {
 			dns_req_free(result);
 			return NULL;
