@@ -39,7 +39,14 @@
 #include <netinet/in.h>
 #include <ldns/ldns.h>
 #include "getdns/getdns.h"
+#include "getdns/getdns_extra.h"
+#include "util/rbtree.h"
+
+
 struct getdns_context;
+struct getdns_upstreams;
+struct getdns_upstream;
+
 
 /**
  * \defgroup strings String Constants
@@ -49,7 +56,6 @@ struct getdns_context;
 #define GETDNS_STR_IPV6 "IPv6"
 #define GETDNS_STR_ADDRESS_TYPE "address_type"
 #define GETDNS_STR_ADDRESS_DATA "address_data"
-#define GETDNS_STR_ADDRESS_STRING "address_string"
 #define GETDNS_STR_PORT "port"
 #define GETDNS_STR_EXTENSION_RETURN_BOTH_V4_AND_V6 "return_both_v4_and_v6"
 
@@ -93,6 +99,8 @@ struct getdns_context;
 #define GETDNS_STR_KEY_NSCOUNT "nscount"
 #define GETDNS_STR_KEY_ARCOUNT "arcount"
 
+#define TIMEOUT_FOREVER ((int64_t)-1)
+#define ASSERT_UNREACHABLE 0
 
 /** @}
  */
@@ -123,6 +131,9 @@ struct mem_funcs {
     mf_union mf;
 };
 
+struct mem_funcs *
+priv_getdns_context_mf(getdns_context *context);
+
 typedef enum network_req_state_enum
 {
 	NET_REQ_NOT_SENT,
@@ -140,11 +151,28 @@ typedef struct getdns_extension_format
 	getdns_data_type exttype;
 } getdns_extension_format;
 
+
+/* State for async tcp stub resolving */
+typedef struct getdns_tcp_state {
+
+	uint8_t *write_buf;
+	size_t   write_buf_len;
+	size_t   written;
+
+	uint8_t *read_buf;
+	size_t   read_buf_len;
+	uint8_t *read_pos;
+	size_t   to_read;
+
+} getdns_tcp_state;
+
 /**
- * Request data for unbound
+ * Request data
  **/
 typedef struct getdns_network_req
 {
+	/* For storage in upstream->netreq_by_query_id */
+	getdns_rbnode_t node;
 	/* the async_id from unbound */
 	int unbound_id;
 	/* state var */
@@ -165,14 +193,28 @@ typedef struct getdns_network_req
 
 	/* next request to issue after this one */
 	struct getdns_network_req *next;
+
+	/* For stub resolving */
+	struct getdns_upstream *upstream;
+	int                     fd;
+	getdns_eventloop_event  event;
+	getdns_tcp_state        tcp;
+	uint16_t                query_id;
+
+	uint16_t                max_udp_payload_size;
+
+	/* Network requests scheduled to write after me */
+	struct getdns_network_req *write_queue_tail;
+
 } getdns_network_req;
 
 /**
  * dns request - manages a number of network requests and
  * the initial data passed to getdns_general
  */
-typedef struct getdns_dns_req
-{
+typedef struct getdns_dns_req {
+	/* For storage in context->outbound_requests */
+	getdns_rbnode_t node;
 
 	/* name */
 	char *name;
@@ -192,6 +234,9 @@ typedef struct getdns_dns_req
 	/* request extensions */
 	struct getdns_dict *extensions;
 
+	/* event loop */
+	getdns_eventloop *loop;
+
 	/* callback data */
 	getdns_callback_t user_callback;
 	void *user_pointer;
@@ -199,14 +244,17 @@ typedef struct getdns_dns_req
 	/* the transaction id */
 	getdns_transaction_t trans_id;
 
-    /* local timeout id */
-    getdns_transaction_t local_timeout_id;
+	/* for scheduling timeouts when using libunbound */
+	getdns_eventloop_event timeout;
 
-    /* dnssec status */
-    int return_dnssec_status;
+	/* dnssec status */
+	int return_dnssec_status;
 
-    /* mem funcs */
-    struct mem_funcs my_mf;
+	/* mem funcs */
+	struct mem_funcs my_mf;
+
+	/* Stuff for stub resolving */
+	struct getdns_upstreams *upstreams;
 
 } getdns_dns_req;
 
@@ -229,6 +277,17 @@ typedef struct getdns_dns_req
     : ((*(obj).mf.ext.free)((obj).mf_arg, (ptr))) \
     )
 
+#define GETDNS_NULL_FREE(obj, ptr)	\
+	do { \
+		if (!(ptr)) \
+			break; \
+		if ((obj).mf_arg == MF_PLAIN) \
+			(*(obj).mf.pln.free)(              (ptr)); \
+		else \
+			(*(obj).mf.ext.free)((obj).mf_arg, (ptr)); \
+		(ptr) = NULL; \
+	} while (0);
+
 #define GETDNS_MALLOC(obj, type)	GETDNS_XMALLOC(obj, type, 1)
 #define GETDNS_REALLOC(obj, ptr, type)	GETDNS_XREALLOC(obj, ptr, type, 1);
 
@@ -243,12 +302,10 @@ getdns_network_req *network_req_new(getdns_dns_req * owner,
     uint16_t request_class, struct getdns_dict *extensions);
 
 /* dns request utils */
-getdns_dns_req *dns_req_new(struct getdns_context *context,
-    const char *name, uint16_t request_type, struct getdns_dict *extensions);
+getdns_dns_req *dns_req_new(getdns_context *context, getdns_eventloop *loop,
+    const char *name, uint16_t request_type, getdns_dict *extensions);
 
 void dns_req_free(getdns_dns_req * req);
 
-
 #endif
-
 /* types-internal.h */
