@@ -476,21 +476,27 @@ create_reply_dict(struct getdns_context *context, getdns_network_req * req,
             }
         }
 
-    	/* other stuff */
+    	/* other stuff
+	 * Note that spec doesn't explicitely mention these.
+	 * They are only showcased in the response dict example */
     	r = getdns_dict_set_int(result, GETDNS_STR_KEY_ANSWER_TYPE,
     	    GETDNS_NAMETYPE_DNS);
         if (r != GETDNS_RETURN_GOOD) {
             break;
         }
     	question = ldns_rr_list_rr(ldns_pkt_question(reply), 0);
-    	name = convert_rdf_to_str(ldns_rr_owner(question));
-    	if (! name) {
-    		r = GETDNS_RETURN_MEMORY_ERROR;
-		break;
+	/* Some packets (like with rcode REFUSED)
+	 * don't even have RRs in the question section! */
+	if (question) {
+		name = convert_rdf_to_str(ldns_rr_owner(question));
+		if (! name) {
+			r = GETDNS_RETURN_MEMORY_ERROR;
+			break;
+		}
+		r = getdns_dict_util_set_string(result,
+		    GETDNS_STR_KEY_CANONICAL_NM, name);
+		free(name);
 	}
-    	r = getdns_dict_util_set_string(result,
-    	    GETDNS_STR_KEY_CANONICAL_NM, name);
-    	free(name);
 
     } while (0);
 
@@ -709,6 +715,80 @@ create_getdns_response(struct getdns_dns_req * completed_request)
 	return result;
 }
 
+/*This method can be used when e.g. a local lookup has been performed and the 
+   result is simply a list of addresses (not a DNS packet)*/
+struct getdns_dict *
+create_getdns_response_from_rr_list(struct getdns_dns_req * completed_request,
+                                    ldns_rr_list * response_list)
+{
+	struct getdns_dict *result = getdns_dict_create_with_context(completed_request->context);
+	struct getdns_list *replies_full = getdns_list_create_with_context(
+	    completed_request->context);
+	struct getdns_list *replies_tree = getdns_list_create_with_context(
+	    completed_request->context);
+	struct getdns_list *just_addrs = NULL;
+	char *canonical_name = NULL;
+	getdns_return_t r = 0;
+
+	/* NOTE: With DNS packet, we ignore any DNSSEC related extensions since we 
+	   don't populate the replies full or tree at all*/
+
+	just_addrs = getdns_list_create_with_context(completed_request->context);
+
+	do {
+		canonical_name = get_canonical_name(completed_request->name);
+		r = getdns_dict_util_set_string(result, GETDNS_STR_KEY_CANONICAL_NM,
+			canonical_name);
+		free(canonical_name);
+		if (r != GETDNS_RETURN_GOOD) {
+			break;
+		}
+
+		/* For local lookups we don't set an answer_type as there isn't a
+		suitable one*/
+
+		r = add_only_addresses(just_addrs, response_list);
+		if (r != GETDNS_RETURN_GOOD) {
+			break;
+		}
+           
+		if (r != GETDNS_RETURN_GOOD)
+			break;
+
+		r = getdns_dict_set_list(result, GETDNS_STR_KEY_REPLIES_TREE,
+			replies_tree);
+		if (r != GETDNS_RETURN_GOOD)
+            break;
+
+		r = getdns_dict_set_list(result, GETDNS_STR_KEY_REPLIES_FULL,
+			replies_full);
+		if (r != GETDNS_RETURN_GOOD)
+			break;
+
+		r = getdns_dict_set_list(result, GETDNS_STR_KEY_JUST_ADDRS,
+			just_addrs);
+		if (r != GETDNS_RETURN_GOOD) {
+            break;
+		}
+
+		r = getdns_dict_set_int(result, GETDNS_STR_KEY_STATUS,
+				GETDNS_RESPSTATUS_GOOD);
+	} while (0);
+
+	/* cleanup */
+	getdns_list_destroy(replies_tree);
+	getdns_list_destroy(replies_full);
+	getdns_list_destroy(just_addrs);
+
+	if (r != 0) {
+		getdns_dict_destroy(result);
+		result = NULL;
+	}
+
+	return result;
+}
+
+
 /**
  * reverse an IP address for PTR lookup
  * @param address_data IP address to reverse
@@ -756,9 +836,9 @@ validate_extensions(struct getdns_dict * extensions)
 	getdns_extension_format *extformat;
 
 	if (extensions)
-		LDNS_RBTREE_FOR(item, struct getdns_dict_item *,
-		    &(extensions->root))
-		{
+		RBTREE_FOR(item, struct getdns_dict_item *,
+		    &(extensions->root)) {
+
 			getdns_extension_format key;
 			key.extstring = (char *) item->node.key;
 			extformat = bsearch(&key, extformats,
